/* SPDX-License-Identifier: GPL-2.0 */
/*
 * periodicstats.h - definitions used in periodic statistics
 *
 * Copyright (C) 2020 iopsys Software Solutions AB. All rights reserved.
 *
 * Author: Shubham Sharma <shubham.sharma@iopsys.eu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 */

#ifndef PERIODICSTATS_H
#define PERIODICSTATS_H

#include <string.h>
#include <time.h>
#include <uci.h>
#include <pthread.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <json-c/json.h>

/* logging though syslog */
#include <syslog.h>

/* To utalize kernel list implementation. */
#include <libubox/list.h>

/* UCI definitions, used to parse the config files at their right location */
#define UCI_CONFIG_PSTATS   "periodicstats"

#define MAX_DATA_STR 4096
#define MAX_REF_STR 256
#define MAX_NAME_STR 128
#define MAX_ALIAS_STR 64
#define MAX_TIME_STR 25

#define DEFAULT_REF_TIME "0001-01-01T00:00:00Z"

#define SAMPLE_SET_DIR "/etc/periodicstats/"

/*
 * Calculation mode enumerator
 * */
enum cal_mode {
	LATEST,
	MINIMUM,
	MAXIMUM,
	AVERAGE
};

/*
 * Structure for parameters inside each sample set
 * */
typedef struct parameters {
	bool enable;
	enum cal_mode param_cal_mode;
	char name[MAX_NAME_STR];
	char reference[MAX_REF_STR];
	char values[MAX_REF_STR];
	struct list_head list;
} param;

/*
 * Structure for sample sets inside global periodic stats
 * */
typedef struct sample_sets {
	bool enable;
	uint32_t smpl_intrvl;
	uint32_t rprt_smpls;
	uint32_t fetch_samples;
	time_t time_ref;
	char name[MAX_NAME_STR];
	struct list_head params;
	struct list_head list;
} smpl_set;

// Global base interval
extern int base_interval;

int pstats_global_init(struct list_head *ss_list);
void pstats_get_stats(struct list_head *global_list);

#endif /* PERIODICSTATS_H */
